@@ -21,11 +21,24 @@ impl NodeHandle {
         NodeHandle(node)
     }
 
-    /// Returns the node that is associated to this specific handle
+    /// Returns a reference to the node that is associated to this specific handle
+    ///
+    /// It is an error to pass in the wrong parser.
+    /// It will either return `None` if this index points outside of the nodes table,
+    /// or it will return the one it points to.
     pub fn get<'p, 'buf>(&self, parser: &'p Parser<'buf>) -> Option<&'p Node<'buf>> {
         parser.resolve_node_id(self.0)
     }
 
+    /// Returns a mutable reference to the node that is associated to this specific handle
+    ///
+    /// It is an error to pass in the wrong parser.
+    /// It will either return `None` if this index points outside of the nodes table,
+    /// or it will return the one it points to.
+    pub fn get_mut<'p, 'buf>(&self, parser: &'p mut Parser<'buf>) -> Option<&'p mut Node<'buf>> {
+        parser.resolve_node_id_mut(self.0)
+    }
+
     /// Returns the internal unique Node ID that maps to a specific node in the node table
     #[inline]
     pub fn get_inner(&self) -> InnerNodeHandle {
