@@ -10,12 +10,12 @@ fn force_as_tag<'a, 'b>(actual: &'a Node<'b>) -> &'a HTMLTag<'b> {
 
 #[test]
 fn inner_html() {
-    let dom = parse("abc <p>test</p> def", ParserOptions::default());
+    let dom = parse("abc <p>test</p> def", ParserOptions::default()).unwrap();
     let parser = dom.parser();
 
     let tag = force_as_tag(dom.children()[1].get(parser).unwrap());
 
-    assert_eq!(tag.inner_html().as_utf8_str(), "<p>test</p>");
+    assert_eq!(tag.inner_html(parser), "<p>test</p>");
 }
 
 #[test]
@@ -23,7 +23,8 @@ fn children_len() {
     let dom = parse(
         "<!-- element 1 --> <div><div>element 3</div></div>",
         ParserOptions::default(),
-    );
+    )
+    .unwrap();
     assert_eq!(dom.children().len(), 2);
 }
 
@@ -32,13 +33,15 @@ fn get_element_by_id_default() {
     let dom = parse(
         "<div></div><p id=\"test\"></p><p></p>",
         ParserOptions::default(),
-    );
+    )
+    .unwrap();
+    let parser = dom.parser();
 
     let tag = dom.get_element_by_id("test").expect("Element not present");
 
     let el = force_as_tag(tag.get(dom.parser()).unwrap());
 
-    assert_eq!(el.inner_html().as_utf8_str(), "<p id=\"test\"></p>")
+    assert_eq!(el.inner_html(parser), "<p id=\"test\"></p>")
 }
 
 #[test]
@@ -46,13 +49,15 @@ fn get_element_by_id_tracking() {
     let dom = parse(
         "<div></div><p id=\"test\"></p><p></p>",
         ParserOptions::default().track_ids(),
-    );
+    )
+    .unwrap();
+    let parser = dom.parser();
 
     let tag = dom.get_element_by_id("test").expect("Element not present");
 
     let el = force_as_tag(tag.get(dom.parser()).unwrap());
 
-    assert_eq!(el.inner_html().as_utf8_str(), "<p id=\"test\"></p>")
+    assert_eq!(el.inner_html(parser), "<p id=\"test\"></p>")
 }
 
 #[test]
@@ -60,7 +65,8 @@ fn get_element_by_class_name_default() {
     let dom = parse(
         "<div></div><p class=\"a b\">hey</p><p></p>",
         ParserOptions::default(),
-    );
+    )
+    .unwrap();
 
     let tag = dom.get_elements_by_class_name("a").next().unwrap();
 
@@ -74,7 +80,8 @@ fn get_element_by_class_name_tracking() {
     let dom = parse(
         "<div></div><p class=\"a b\">hey</p><p></p>",
         ParserOptions::default().track_ids(),
-    );
+    )
+    .unwrap();
 
     let tag = dom.get_elements_by_class_name("a").next().unwrap();
 
@@ -85,7 +92,7 @@ fn get_element_by_class_name_tracking() {
 
 #[test]
 fn html5() {
-    let dom = parse("<!DOCTYPE html> hello", ParserOptions::default());
+    let dom = parse("<!DOCTYPE html> hello", ParserOptions::default()).unwrap();
 
     assert_eq!(dom.version(), Some(HTMLVersion::HTML5));
     assert_eq!(dom.children().len(), 1)
@@ -96,7 +103,8 @@ fn nested_inner_text() {
     let dom = parse(
         "<p>hello <p>nested element</p></p>",
         ParserOptions::default(),
-    );
+    )
+    .unwrap();
     let parser = dom.parser();
 
     let el = force_as_tag(dom.children()[0].get(parser).unwrap());
@@ -108,7 +116,7 @@ fn nested_inner_text() {
 fn owned_dom() {
     let owned_dom = {
         let input = String::from("<p id=\"test\">hello</p>");
-        let dom = unsafe { parse_owned(input, ParserOptions::default()) };
+        let dom = unsafe { parse_owned(input, ParserOptions::default()).unwrap() };
         dom
     };
 
@@ -124,7 +132,7 @@ fn owned_dom() {
 fn move_owned() {
     let input = String::from("<p id=\"test\">hello</p>");
 
-    let guard = unsafe { parse_owned(input, ParserOptions::default()) };
+    let guard = unsafe { parse_owned(input, ParserOptions::default()).unwrap() };
 
     fn move_me<T>(p: T) -> T {
         p
@@ -145,7 +153,7 @@ fn move_owned() {
 fn with() {
     let input = r#"<p>hello <span>whats up</span></p>"#;
 
-    let dom = parse(input, ParserOptions::default());
+    let dom = parse(input, ParserOptions::default()).unwrap();
     let parser = dom.parser();
 
     let tag = dom
@@ -162,13 +170,13 @@ fn with() {
 #[test]
 fn abrupt_attributes_stop() {
     let input = r#"<p "#;
-    parse(input, ParserOptions::default());
+    parse(input, ParserOptions::default()).unwrap();
 }
 
 #[test]
 fn dom_nodes() {
     let input = r#"<p><p><a>nested</a></p></p>"#;
-    let dom = parse(input, ParserOptions::default());
+    let dom = parse(input, ParserOptions::default()).unwrap();
     let parser = dom.parser();
     let element = dom
         .nodes()
@@ -182,17 +190,50 @@ fn dom_nodes() {
 fn fuzz() {
     // Some tests that would previously panic or end in an infinite loop
     // We don't need to assert anything here, just see that they finish
-    parse("J\x00<", ParserOptions::default());
-    parse("<!J", ParserOptions::default());
+    parse("J\x00<", ParserOptions::default()).unwrap();
+    parse("<!J", ParserOptions::default()).unwrap();
 
     // Miri is too slow... :(
     let count = if cfg!(miri) { 100usize } else { 10000usize };
 
-    parse(&"<p>".repeat(count), ParserOptions::default());
+    parse(&"<p>".repeat(count), ParserOptions::default()).unwrap();
+}
+
+#[test]
+fn query_selector_simple() {
+    let input = "<div><p class=\"hi\">hello</p></div>";
+    let dom = parse(input, ParserOptions::default()).unwrap();
+    let parser = dom.parser();
+    let mut selector = dom.query_selector(".hi").unwrap();
+    let el = force_as_tag(selector.next().and_then(|x| x.get(parser)).unwrap());
+
+    assert_eq!(dom.nodes().len(), 3);
+    assert_eq!(el.inner_text(parser), "hello");
+}
+
+#[test]
+fn mutate_dom() {
+    let input = r#"<img src="test.png" />"#;
+    let mut dom = parse(input, ParserOptions::default()).unwrap();
+
+    let mut selector = dom.query_selector("[src]").unwrap();
+    let handle = selector.next().unwrap();
+
+    let parser = dom.parser_mut();
+
+    let el = handle.get_mut(parser).unwrap();
+    let tag = el.as_tag_mut().unwrap();
+    let attr = tag.attributes_mut();
+    let bytes = attr.get_mut("src").flatten().unwrap();
+    bytes.set("world.png").unwrap();
+
+    assert_eq!(attr.get("src"), Some(Some("world.png".into())));
 }
 
 #[cfg(feature = "simd")]
 mod simd {
+    // These tests make sure that SIMD functions do the right thing
+
     use crate::util;
 
     #[test]
@@ -208,8 +249,10 @@ mod simd {
         assert_eq!(util::find_fast(b"abcdefghiabcdefghi .", b' '), Some(18));
         assert_eq!(util::find_fast(b"abcdefghiabcdefghi.", b' '), None);
 
-        let long = "a".repeat(100000) + "b";
-        assert_eq!(util::find_fast(long.as_bytes(), b'b'), Some(100000));
+        let count = if cfg!(miri) { 500usize } else { 1000usize };
+
+        let long = "a".repeat(count) + "b";
+        assert_eq!(util::find_fast(long.as_bytes(), b'b'), Some(count));
     }
 
     #[test]
@@ -252,16 +295,123 @@ mod simd {
     }
 }
 
-#[test]
-fn query_selector_simple() {
-    let input = "<div><p class=\"hi\">hello</p></div>";
-    let dom = parse(input, ParserOptions::default());
-    let parser = dom.parser();
-    let mut selector = dom.query_selector(".hi").unwrap();
-    let el = force_as_tag(selector.next().and_then(|x| x.get(parser)).unwrap());
+mod bytes {
+    use crate::bytes::*;
 
-    assert_eq!(dom.nodes().len(), 3);
-    assert_eq!(el.inner_text(parser), "hello");
+    #[test]
+    fn from_str() {
+        let x = Bytes::from("hello");
+        assert_eq!(x.as_bytes(), b"hello");
+    }
+
+    #[test]
+    fn from_bytes() {
+        let x = Bytes::from(b"hello" as &[u8]);
+        assert_eq!(x.as_bytes(), b"hello");
+    }
+
+    #[test]
+    fn as_bytes_borrowed() {
+        let xb = Bytes::from(b"hello" as &[u8]);
+        assert_eq!(xb.as_bytes_borrowed(), Some(b"hello" as &[u8]));
+
+        let mut xc = xb.clone();
+        xc.set("test2").unwrap();
+        assert_eq!(xc.as_bytes_borrowed(), None);
+    }
+
+    #[test]
+    fn as_utf8_str() {
+        assert_eq!(Bytes::from("hello").as_utf8_str(), "hello");
+    }
+
+    #[test]
+    fn clone_shallow() {
+        // cloning a borrowed slice does not deep-clone
+        let x = Bytes::from("hello");
+        let xp = x.as_ptr();
+
+        let y = x.clone();
+        let yp = y.as_ptr();
+
+        assert_eq!(xp, yp);
+    }
+
+    #[test]
+    fn drop_old_owned() {
+        let mut x = Bytes::from("");
+        x.set("test").unwrap();
+        x.set("test2").unwrap();
+    }
+
+    #[test]
+    fn clone_owned_deep() {
+        let mut x = Bytes::from("");
+        x.set("hello").unwrap();
+        let xp = x.as_ptr();
+
+        let y = x.clone();
+        let yp = y.as_ptr();
+
+        assert_eq!(x, y);
+        assert_ne!(xp, yp);
+    }
+
+    #[test]
+    fn empty() {
+        let _x = Bytes::new();
+    }
+
+    #[test]
+    fn empty_set() {
+        let mut x = Bytes::new();
+        x.set("hello").unwrap();
+    }
+
+    #[test]
+    fn set() {
+        let mut x = Bytes::from("hello");
+        let xp = x.as_ptr();
+
+        x.set("world").unwrap();
+        let xp2 = x.as_ptr();
+
+        // check that the changes are reflected
+        assert_eq!(x.as_bytes(), b"world");
+
+        // pointer must be different now as the call to `set` should cause an allocation
+        assert_ne!(xp, xp2);
+    }
+
+    #[test]
+    fn clone_deep() {
+        let x = Bytes::from("hello");
+        let xp = x.as_ptr();
+
+        let mut y = x.clone();
+        y.set("world").unwrap();
+        let yp = y.as_ptr();
+
+        assert_ne!(xp, yp);
+    }
+
+    #[test]
+    fn into_owned_bytes() {
+        let mut x1 = Bytes::new();
+        x1.set("hello").unwrap(); // &str
+
+        let mut x2 = x1.clone();
+        x2.set(b"world" as &[u8]).unwrap(); // &[u8]
+
+        let mut x3 = x1.clone();
+        x3.set(vec![0u8, 1, 2, 3, 4]).unwrap(); // Vec<u8>
+
+        let mut x4 = x1.clone();
+        x4.set(vec![0u8, 1, 2, 3, 4].into_boxed_slice()).unwrap(); // Box<[u8]>
+
+        let mut x5 = x1.clone();
+        x5.set(String::from("Tests are important")).unwrap(); // String
+    }
 }
 
 #[test]
@@ -273,7 +423,7 @@ fn valueless_attribute() {
         </a>
     "#;
 
-    let dom = parse(input, ParserOptions::default());
+    let dom = parse(input, ParserOptions::default()).unwrap();
     let element = dom.get_element_by_id("u54423");
 
     assert!(element.is_some());
@@ -286,7 +436,7 @@ fn unquoted() {
         <a id=u54423>Hello World</a>
     "#;
 
-    let dom = parse(input, ParserOptions::default());
+    let dom = parse(input, ParserOptions::default()).unwrap();
     let parser = dom.parser();
     let element = dom.get_element_by_id("u54423");
 
